searchData={"items":[{"type":"behaviour","title":"Horde.Process","doc":"A module that uses `Horde.Process` is a `GenServer` process that is managed via Horde. Horde processes are started and supervised by a Horde Supervisor and registered with a Horde Registry. Many boilerplate functions providing interactions with Horde are automatically imported by this module when you `use Horde.Process`.\n\nRequired options for `use Horde.Process` are as follows:\n\n- `:supervisor` - The name of the Horde Supervisor to use, e.g. `MyApp.HordeSupervisor`.\n- `:registry` - The name of the Horde Registry to use, e.g. `MyApp.HordeRegistry`.\n\nAdditionally, all Horde Process modules *MUST* implement the following functions:\n\n- `process_id/1` which should return a unique process identifier. This is used for process lookup and registration.\n- `child_spec/1` returns the child spec used by the supervisor to start the process.\n\nThe above functions are defined via `@callback` to ensure compiler feedback on the implementation of these functions.","ref":"Horde.Process.html"},{"type":"behaviour","title":"Horde Registry Notice - Horde.Process","doc":"When implementing a Horde Process, it is *CRITICAL* to use `{:continue, term}` in the `init/1` callback whenever non-trivial initialization is required. Horde CANNOT add the process to the registry until `init/1` completes. The longer it takes `init/1` to finish, the longer before the process can enter the registry and be used.\n\nThis will have a knock-on effect of causing `fetch/1` to return `nil` even though `start/1` will return `:ignore`. To reduce the likelihood of this scenario, it's recommended to use `{:continue, term}` in `init/1` to ensure the process is started and registered as quickly as possible, then performing all non-trivial initialization in `handle_continue/2`.\n\nEven if you have the fastest `init/1` ever, there's still a possibility - within a distributed system - for Horde to be unable to fetch a process but also unable to start the process. This is because the local node ensures that you can't start a second process with the same unique process id, but the Horde registry is eventually consistent and so remote nodes won't know if a process is starting up elsewhere. And so this module implements \"wait functions\" to help retry the entire process a set number of times over a set period of time. It isn't guaranteed to always work, but it's configurable so that you can make it work best for your application.","ref":"Horde.Process.html#module-horde-registry-notice"},{"type":"behaviour","title":"Example - Horde.Process","doc":"If you wanted to use this module to create a very simple Horde Process, you could do the following:\n\n```elixir\ndefmodule MyApp.User.Process do\n  use Horde.Process, supervisor: MyApp.User.HordeSupervisor, registry: MyApp.User.HordeRegistry\n\n  @impl Horde.Process\n  def process_id(%{\"user_id\" => user_id}), do: user_id\n  def process_id(%{user_id: user_id}), do: user_id\n  def process_id(user_id) when is_binary(user_id), do: user_id\n\n  @impl Horde.Process\n  def child_spec(user_id) do\n    %{\n      id: user_id,\n      start: {__MODULE__, :start_link, [user_id]},\n      restart: :transient,\n      shutdown: 10_000\n    }\n  end\n\n  @impl GenServer\n  # Set up the process state quickly and have `handle_continue/2` do the rest.\n  def init(user_id) do\n    Process.flag(:trap_exit, true)\n    {:ok, user_id, {:continue, :init}}\n  end\n\n  @impl GenServer\n  def handle_continue(:init, user_id) do\n    {:ok, user} = MyApp.User.fetch(user_id)\n    {:noreply, user}\n  end\nend\n```\n\nThe supervisor and registry must be started appropriately by your application to be used by the Horde Process.\n\nWe can see that `process_id/1` can convert a map or a binary string to a user id, and that `child_spec/1` takes in that user id to return a child process specification.\n\nWe can also see the use of `{:continue, :init}` to ensure `init/1` completes as quickly as possible. Since we're loading the user from the database, we don't want to block the process from being registered with the Horde registry until that's done. Instead, we set the process state to `user_id` temporarily and pass along `{:continue, :init}` so that we know to take that user id and grab the full user schema from the database in `handle_continue/2`.\n\nYou can then implement `handle_call/3` and `handle_cast/2` as you would with any other `GenServer` module. You may either invoke these directly or use the imported functions provided by `Horde.Process`, e.g. `MyApp.User.Process.call!/2`.","ref":"Horde.Process.html#module-example"},{"type":"macro","title":"Horde.Process.__using__/1","doc":"When you `use Horde.Process` you must specify *at least* the following options:\n\n- `:supervisor` - The name of the Horde Supervisor to use, e.g. `MyApp.HordeSupervisor`.\n- `:registry` - The name of the Horde Registry to use, e.g. `MyApp.HordeRegistry`.\n\nAnd you may optionally specify:\n\n- `:wait_sleep` - The number of milliseconds to sleep between attempts to fetch a process from the registry. Defaults to `100`.\n- `:wait_max` - The maximum number of attempts to fetch a process from the registry. Defaults to `5`.\n\nIn addition to generating functions that are tied to the given Horde Supervisor and Registry modules, this macro also performs  `use GenServer` and `@behavior Horde.Process`.","ref":"Horde.Process.html#__using__/1"},{"type":"macro","title":"Wait Options - Horde.Process.__using__/1","doc":"The `:wait_*` options are used by `wait_for_init/2` to determine how long to wait for a process to be registered before giving up and returning `{:error, :not_found}`. Since the registry is eventually consistent, it's possible to fetch a process that can't be started because it's already in the middle of starting, but also which has not yet been added to (or replicated by) the Horde Registry. This is a common scenario when using `start/1` and/or `fetch/1` at the same time from different nodes in a distributed system.\n\nThe maximum amount of time that `wait_for_init/2` will wait before failing is `wait_sleep * wait_max` milliseconds, which defaults to 500 milliseconds. If your application is creating and registering processes quickly, you can try decreasing `:wait_sleep` while increasing `:wait_max`; this means retries will happen more frequently. If, however, you application is creating and registering processes slowly, you can try increasing `:wait_sleep` to avoid unnecessary retries. If you prefer to fail quickly when a process is not registered, you can decrease `:wait_max` to a lower number.","ref":"Horde.Process.html#__using__/1-wait-options"},{"type":"callback","title":"Horde.Process.child_spec/1","doc":"Given a unique process id, returns a child spec that will be used by the Horde Supervisor to start the process.\n\nWhen using the functions provided by `Horde.Process`, the process id given will come from `process_id/1`.","ref":"Horde.Process.html#c:child_spec/1"},{"type":"callback","title":"Horde.Process.process_id/1","doc":"Given an arbitrary argument, return a unique process identifier. This will, among other things, be used to register the process with a Horde Registry.","ref":"Horde.Process.html#c:process_id/1"}],"content_type":"text/markdown"}